# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'layout3.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(610, 433)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/myIcons/plots.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.myTabs = QtWidgets.QTabWidget(self.centralwidget)
        self.myTabs.setObjectName("myTabs")
        self.browserTab = QtWidgets.QWidget()
        self.browserTab.setObjectName("browserTab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.browserTab)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.webFrame = QtWidgets.QFrame(self.browserTab)
        self.webFrame.setMinimumSize(QtCore.QSize(50, 0))
        self.webFrame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.webFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.webFrame.setObjectName("webFrame")
        self.webLayout = QtWidgets.QVBoxLayout(self.webFrame)
        self.webLayout.setContentsMargins(0, 0, 0, 0)
        self.webLayout.setSpacing(0)
        self.webLayout.setObjectName("webLayout")
        self.verticalLayout_2.addWidget(self.webFrame)
        self.myTabs.addTab(self.browserTab, "")
        self.toc = QtWidgets.QWidget()
        self.toc.setObjectName("toc")
        self.experimentListLayout2 = QtWidgets.QVBoxLayout(self.toc)
        self.experimentListLayout2.setContentsMargins(0, 0, 0, 0)
        self.experimentListLayout2.setSpacing(0)
        self.experimentListLayout2.setObjectName("experimentListLayout2")
        self.scrollArea = QtWidgets.QScrollArea(self.toc)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents_2 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_2.setGeometry(QtCore.QRect(0, 0, 604, 356))
        self.scrollAreaWidgetContents_2.setObjectName("scrollAreaWidgetContents_2")
        self.experimentListLayout = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents_2)
        self.experimentListLayout.setContentsMargins(0, 0, 0, 0)
        self.experimentListLayout.setSpacing(2)
        self.experimentListLayout.setObjectName("experimentListLayout")
        self.scrollArea.setWidget(self.scrollAreaWidgetContents_2)
        self.experimentListLayout2.addWidget(self.scrollArea)
        self.myTabs.addTab(self.toc, "")
        self.codeTab = QtWidgets.QWidget()
        self.codeTab.setObjectName("codeTab")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.codeTab)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.codeBrowser = QtWidgets.QTextBrowser(self.codeTab)
        self.codeBrowser.setDocumentTitle("")
        self.codeBrowser.setLineWrapMode(QtWidgets.QTextEdit.NoWrap)
        self.codeBrowser.setObjectName("codeBrowser")
        self.verticalLayout_3.addWidget(self.codeBrowser)
        self.myTabs.addTab(self.codeTab, "")
        self.remoteExptTab = QtWidgets.QWidget()
        self.remoteExptTab.setObjectName("remoteExptTab")
        self.exptLayout = QtWidgets.QVBoxLayout(self.remoteExptTab)
        self.exptLayout.setContentsMargins(0, 0, 0, 0)
        self.exptLayout.setSpacing(0)
        self.exptLayout.setObjectName("exptLayout")
        self.myTabs.addTab(self.remoteExptTab, "")
        self.verticalLayout_4.addWidget(self.myTabs)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 610, 22))
        self.menubar.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.menubar.setObjectName("menubar")
        self.menuTools = QtWidgets.QMenu(self.menubar)
        self.menuTools.setObjectName("menuTools")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionLogin_to_Eyes_Online = QtWidgets.QAction(MainWindow)
        self.actionLogin_to_Eyes_Online.setObjectName("actionLogin_to_Eyes_Online")
        self.actionFast_Upload_Graphs = QtWidgets.QAction(MainWindow)
        self.actionFast_Upload_Graphs.setCheckable(True)
        self.actionFast_Upload_Graphs.setObjectName("actionFast_Upload_Graphs")
        self.actionJoin_Room = QtWidgets.QAction(MainWindow)
        self.actionJoin_Room.setObjectName("actionJoin_Room")
        self.actionLeave_Room = QtWidgets.QAction(MainWindow)
        self.actionLeave_Room.setObjectName("actionLeave_Room")
        self.menuTools.addAction(self.actionLogin_to_Eyes_Online)
        self.menuTools.addAction(self.actionFast_Upload_Graphs)
        self.menuTools.addAction(self.actionJoin_Room)
        self.menuTools.addAction(self.actionLeave_Room)
        self.menubar.addAction(self.menuTools.menuAction())

        self.retranslateUi(MainWindow)
        self.myTabs.setCurrentIndex(1)
        self.actionLogin_to_Eyes_Online.triggered.connect(MainWindow.showLoginScreen)
        self.actionFast_Upload_Graphs.toggled['bool'].connect(MainWindow.setAutoGraph)
        self.actionJoin_Room.triggered.connect(MainWindow.showJoinRoom)
        self.actionLeave_Room.triggered.connect(MainWindow.showLeaveRoom)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "CSpark Virtual Labs"))
        self.myTabs.setTabText(self.myTabs.indexOf(self.browserTab), _translate("MainWindow", "Browser"))
        self.myTabs.setTabText(self.myTabs.indexOf(self.toc), _translate("MainWindow", "Experiment List"))
        self.codeBrowser.setPlaceholderText(_translate("MainWindow", "Experiment source codes will be shown here..."))
        self.myTabs.setTabText(self.myTabs.indexOf(self.codeTab), _translate("MainWindow", "Code"))
        self.myTabs.setTabText(self.myTabs.indexOf(self.remoteExptTab), _translate("MainWindow", "Experiment"))
        self.menuTools.setTitle(_translate("MainWindow", "Tools"))
        self.actionLogin_to_Eyes_Online.setText(_translate("MainWindow", "Login to Eyes Online"))
        self.actionFast_Upload_Graphs.setText(_translate("MainWindow", "Fast Upload Graphs"))
        self.actionJoin_Room.setText(_translate("MainWindow", "Join Room"))
        self.actionLeave_Room.setText(_translate("MainWindow", "Leave Room"))
from . import res_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'layout3.ui'
#
# Created by: PyQt5 UI code generator 5.14.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(610, 433)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.myTabs = QtWidgets.QTabWidget(self.centralwidget)
        self.myTabs.setObjectName("myTabs")
        self.toc = QtWidgets.QWidget()
        self.toc.setObjectName("toc")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.toc)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.webFrame = QtWidgets.QFrame(self.toc)
        self.webFrame.setMinimumSize(QtCore.QSize(50, 0))
        self.webFrame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.webFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.webFrame.setObjectName("webFrame")
        self.webLayout = QtWidgets.QVBoxLayout(self.webFrame)
        self.webLayout.setContentsMargins(0, 0, 0, 0)
        self.webLayout.setSpacing(0)
        self.webLayout.setObjectName("webLayout")
        self.verticalLayout_2.addWidget(self.webFrame)
        self.myTabs.addTab(self.toc, "")
        self.experimentTab = QtWidgets.QWidget()
        self.experimentTab.setObjectName("experimentTab")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.experimentTab)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.frame = QtWidgets.QFrame(self.experimentTab)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.myButton = QtWidgets.QPushButton(self.frame)
        self.myButton.setProperty("remote", True)
        self.myButton.setObjectName("myButton")
        self.gridLayout_2.addWidget(self.myButton, 1, 0, 1, 1)
        self.myButton2 = QtWidgets.QPushButton(self.frame)
        self.myButton2.setProperty("remote", True)
        self.myButton2.setObjectName("myButton2")
        self.gridLayout_2.addWidget(self.myButton2, 1, 1, 1, 1)
        self.timeLabel = QtWidgets.QLabel(self.frame)
        self.timeLabel.setObjectName("timeLabel")
        self.gridLayout_2.addWidget(self.timeLabel, 0, 0, 1, 2)
        self.verticalLayout.addLayout(self.gridLayout_2)
        self.mySlider = QtWidgets.QSlider(self.frame)
        self.mySlider.setMaximum(1000)
        self.mySlider.setOrientation(QtCore.Qt.Horizontal)
        self.mySlider.setProperty("remote", True)
        self.mySlider.setObjectName("mySlider")
        self.verticalLayout.addWidget(self.mySlider)
        self.mySlider2 = QtWidgets.QSlider(self.frame)
        self.mySlider2.setOrientation(QtCore.Qt.Horizontal)
        self.mySlider2.setObjectName("mySlider2")
        self.verticalLayout.addWidget(self.mySlider2)
        self.myCombo = QtWidgets.QComboBox(self.frame)
        self.myCombo.setObjectName("myCombo")
        self.verticalLayout.addWidget(self.myCombo)
        self.fr_1 = QtWidgets.QCheckBox(self.frame)
        self.fr_1.setProperty("remote", True)
        self.fr_1.setObjectName("fr_1")
        self.verticalLayout.addWidget(self.fr_1)
        self.fr_2 = QtWidgets.QCheckBox(self.frame)
        self.fr_2.setObjectName("fr_2")
        self.verticalLayout.addWidget(self.fr_2)
        self.fr_3 = QtWidgets.QCheckBox(self.frame)
        self.fr_3.setObjectName("fr_3")
        self.verticalLayout.addWidget(self.fr_3)
        self.fr_4 = QtWidgets.QCheckBox(self.frame)
        self.fr_4.setObjectName("fr_4")
        self.verticalLayout.addWidget(self.fr_4)
        self.fr_5 = QtWidgets.QCheckBox(self.frame)
        self.fr_5.setObjectName("fr_5")
        self.verticalLayout.addWidget(self.fr_5)
        self.fr_6 = QtWidgets.QCheckBox(self.frame)
        self.fr_6.setObjectName("fr_6")
        self.verticalLayout.addWidget(self.fr_6)
        self.fr_7 = QtWidgets.QCheckBox(self.frame)
        self.fr_7.setObjectName("fr_7")
        self.verticalLayout.addWidget(self.fr_7)
        self.fr_8 = QtWidgets.QCheckBox(self.frame)
        self.fr_8.setObjectName("fr_8")
        self.verticalLayout.addWidget(self.fr_8)
        self.horizontalLayout.addWidget(self.frame)
        self.frame_2 = QtWidgets.QFrame(self.experimentTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_2.sizePolicy().hasHeightForWidth())
        self.frame_2.setSizePolicy(sizePolicy)
        self.frame_2.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.plotArea = QtWidgets.QVBoxLayout(self.frame_2)
        self.plotArea.setContentsMargins(2, 2, 2, -1)
        self.plotArea.setSpacing(2)
        self.plotArea.setObjectName("plotArea")
        self.horizontalLayout.addWidget(self.frame_2)
        self.myTabs.addTab(self.experimentTab, "")
        self.code = QtWidgets.QWidget()
        self.code.setObjectName("code")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.code)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.codeBrowser = QtWidgets.QTextBrowser(self.code)
        self.codeBrowser.setDocumentTitle("")
        self.codeBrowser.setLineWrapMode(QtWidgets.QTextEdit.NoWrap)
        self.codeBrowser.setObjectName("codeBrowser")
        self.verticalLayout_3.addWidget(self.codeBrowser)
        self.myTabs.addTab(self.code, "")
        self.remoteExptTab = QtWidgets.QWidget()
        self.remoteExptTab.setObjectName("remoteExptTab")
        self.exptLayout = QtWidgets.QVBoxLayout(self.remoteExptTab)
        self.exptLayout.setContentsMargins(0, 0, 0, 0)
        self.exptLayout.setSpacing(0)
        self.exptLayout.setObjectName("exptLayout")
        self.myTabs.addTab(self.remoteExptTab, "")
        self.gridLayout.addWidget(self.myTabs, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 610, 22))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        self.myTabs.setCurrentIndex(0)
        self.myButton.clicked.connect(MainWindow.buttonClicked)
        self.myButton2.clicked.connect(MainWindow.button2Clicked)
        self.mySlider.valueChanged['int'].connect(MainWindow.sliderChanged)
        self.mySlider2.valueChanged['int'].connect(MainWindow.slider2Changed)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.myTabs.setTabText(self.myTabs.indexOf(self.toc), _translate("MainWindow", "Contents"))
        self.myButton.setText(_translate("MainWindow", "Button 1"))
        self.myButton2.setText(_translate("MainWindow", "Button 2"))
        self.timeLabel.setText(_translate("MainWindow", "Time Label"))
        self.fr_1.setText(_translate("MainWindow", "Flow 1"))
        self.fr_2.setText(_translate("MainWindow", "Flow 2"))
        self.fr_3.setText(_translate("MainWindow", "Flow 3"))
        self.fr_4.setText(_translate("MainWindow", "Flow 4"))
        self.fr_5.setText(_translate("MainWindow", "Flow 5"))
        self.fr_6.setText(_translate("MainWindow", "Flow 6"))
        self.fr_7.setText(_translate("MainWindow", "Flow 7"))
        self.fr_8.setText(_translate("MainWindow", "Flow 8"))
        self.myTabs.setTabText(self.myTabs.indexOf(self.experimentTab), _translate("MainWindow", "Demo"))
        self.codeBrowser.setPlaceholderText(_translate("MainWindow", "Experiment source codes will be shown here..."))
        self.myTabs.setTabText(self.myTabs.indexOf(self.code), _translate("MainWindow", "Code"))
        self.myTabs.setTabText(self.myTabs.indexOf(self.remoteExptTab), _translate("MainWindow", "Experiment"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'exptEntry.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(583, 34)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        Form.setMinimumSize(QtCore.QSize(100, 30))
        self.horizontalLayout = QtWidgets.QHBoxLayout(Form)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.frame_2 = QtWidgets.QFrame(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_2.sizePolicy().hasHeightForWidth())
        self.frame_2.setSizePolicy(sizePolicy)
        self.frame_2.setMinimumSize(QtCore.QSize(100, 20))
        self.frame_2.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.frame_2)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setHorizontalSpacing(3)
        self.gridLayout_3.setVerticalSpacing(0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.toolButton = QtWidgets.QToolButton(self.frame_2)
        self.toolButton.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/myIcons/play.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.toolButton.setIcon(icon)
        self.toolButton.setObjectName("toolButton")
        self.gridLayout_3.addWidget(self.toolButton, 1, 0, 1, 1)
        self.exptIntro = QtWidgets.QLabel(self.frame_2)
        self.exptIntro.setObjectName("exptIntro")
        self.gridLayout_3.addWidget(self.exptIntro, 1, 2, 1, 1)
        self.exptName = QtWidgets.QLabel(self.frame_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.exptName.sizePolicy().hasHeightForWidth())
        self.exptName.setSizePolicy(sizePolicy)
        self.exptName.setMinimumSize(QtCore.QSize(150, 0))
        self.exptName.setObjectName("exptName")
        self.gridLayout_3.addWidget(self.exptName, 1, 1, 1, 1)
        self.horizontalLayout.addWidget(self.frame_2)

        self.retranslateUi(Form)
        self.toolButton.clicked.connect(Form.play)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.exptIntro.setText(_translate("Form", "intro"))
        self.exptName.setText(_translate("Form", "Name"))
from . import res_rc



from PyQt5 import QtGui,QtCore,QtWidgets
from PyQt5.QtWebChannel import QWebChannel

import os,string,time,sys,base64,hashlib,re,zipfile
from .templates import ui_layout3 as layout
from . import utilities
from .socketcontroller import socketController
from scipy.interpolate import interp1d
from flask import json

import pyqtgraph as pg
import numpy as np


from PyQt5.QtWebEngineWidgets import QWebEngineView, QWebEnginePage
from PyQt5 import QtWebEngineWidgets

import importlib,requests,os

class webPage(QWebEnginePage):
	def __init__(self, *args, **kwargs):
		super(webPage, self).__init__()
		self.featurePermissionRequested.connect(self.onFeaturePermissionRequested)
	def javaScriptConsoleMessage(self,level, msg, line, source):
		print (' \033[4;33m line %d: %s !! %s , %s \033[0m' % ( line, msg, source,level))

	def onFeaturePermissionRequested(self, url, feature):
		print('feature requested',feature)
		if feature in (QWebEnginePage.MediaAudioCapture, 
			QWebEnginePage.MediaVideoCapture, 
			QWebEnginePage.MediaAudioVideoCapture,
			QWebEnginePage.DesktopVideoCapture,
			QWebEnginePage.DesktopAudioVideoCapture):
			self.setFeaturePermission(url, feature, QWebEnginePage.PermissionGrantedByUser)
		else:
			self.setFeaturePermission(url, feature, QWebEnginePage.PermissionDeniedByUser)


	def certificateError(self, certificateError):
		certificateError.isOverridable()
		certificateError.ignoreCertificateError()
		return True

class helpWin(QWebEngineView):
	def __init__(self, parent):
		super(helpWin, self).__init__()
		self.parent=parent
		self.mypage = webPage(self)
		self.setPage(self.mypage)

	def loadPage(self,url):
		print('loading page...',url)
		self.setUrl(QtCore.QUrl(url))
		self.show()




class AppWindow(QtGui.QMainWindow, layout.Ui_MainWindow):
	initSocket = QtCore.pyqtSignal(object,object)
	getScreenshot = QtCore.pyqtSignal(object)
	remoteAccessSignal = QtCore.pyqtSignal(object)
	roomStatusSignal = QtCore.pyqtSignal(object,object)


	def __init__(self, app,**kwargs):
		super(AppWindow, self).__init__(None)
		self.setupUi(self)
		self.remote = kwargs.get('remote',False)

		self.p = None
		self.roomStatusSignal.connect(self.join_room_callback)
		

		self.statusBar = self.statusBar()
		self.chatbox = QtWidgets.QLineEdit(self)
		self.chatbox.setStyleSheet('max-width:200px')
		self.chatbox.setPlaceholderText('chat with admin...')
		self.chatbox.returnPressed.connect(self.sendChatMessage)
		self.statusBar.addPermanentWidget(self.chatbox)

		self.lastTransmittedData = {}
		self.autotransmit = True
		self.autographtransmit = False #Linked to the graph onchange event
		
		## Local Scripts directory for remotely downloaded content
		localstorage = '.expeyesapps'
		homedir = os.path.expanduser('~')
		self.appdir = os.path.join(homedir,localstorage)
		if not os.path.isdir(self.appdir):
			os.makedirs(self.appdir)
		self.newcode =None #New experiment widget. loaded remotely
		self.availableScripts = {}
		self.availableExperiments = {}
		self.loggedIn = False
		self.pname = None

		try:
			with open(os.path.join(self.appdir,'scripts.json'), 'r', encoding='utf-8') as f:
				scripts = json.load(f)
				self.refreshScriptList(scripts)
		except Exception as e:
			print('no scripts found',e)

		if 'remote' in sys.argv or self.remote:
			self.url = 'https://expeyes.scischool.in'
		else:
			self.url = 'http://mypc.in'

		if 'autologin' in sys.argv: QtCore.QTimer.singleShot(0, self.loginScreen ) #Launch the login screen

		self.socklib = socketController(self)

		self.setTheme('aqua')
		self.setupBrowser()

	def showLoginScreen(self):
		QtCore.QTimer.singleShot(0, self.loginScreen ) #Launch the login screen

	def showJoinRoom(self):
		QtCore.QTimer.singleShot(0, self.joinRoomScreen ) #Launch the room joining dialog
	def showLeaveRoom(self):
		QtCore.QTimer.singleShot(0, self.leaveRoomScreen ) #Launch the room leaving dialog

	def refreshScriptList(self,scripts):
		self.availableScripts.update(scripts)
		for a in self.availableExperiments:
			self.availableExperiments[a].setParent(None)
		self.availableExperiments = {}
		#import json
		with open(os.path.join(self.appdir,'scripts.json'), 'w', encoding='utf-8') as f:
			json.dump(scripts, f, ensure_ascii=False, indent=4)
		
		for packagename in scripts:
			pkg = scripts[packagename]
			modname = re.split(r'[\\,/]',pkg['path'])[-1]
			e = utilities.exptEntryHeader(modname,os.path.join(self.appdir,modname),pkg.get('filetype',''))
			self.availableExperiments[modname] = e
			self.experimentListLayout.addWidget(e)

			for expt in pkg['data']:
				e = utilities.exptEntry(expt['Filename'],expt['Intro'],pkg['path'],self.appdir,pkg['hash'],self.downloadAndExecuteExperiment, filetype = pkg.get('filetype',None))
				self.experimentListLayout.addWidget(e)
				self.availableExperiments[expt['Filename']] = e
		

	def loginScreen(self):
		self.room = 'd'
		if len(sys.argv)>1:
			testname = sys.argv[1];	self.name = testname; self.pwd = testname
		else:
			self.name = ''; self.pwd = ''

		self.loginDialog = utilities.loginDialog(self,username = self.name, password = self.pwd,classroom = self.room,url = self.url)

		if 'skip' not in sys.argv: #Command line skipping of the login screen . auto-login. for testing
			retval = self.loginDialog.exec_()
			if not retval:
				return

		try:	# 167.71.230.181 expeyes.scischool.in
			self.name = self.loginDialog.getUsername()
			self.pwd = self.loginDialog.getPassword()
			self.room = self.loginDialog.getClassroom()
			self.url = self.loginDialog.getUrl()
			
			if not self.loggedIn:
				if 'remote' in sys.argv or self.remote:
					self.socklib.connect(self.url,{'name':self.name,'password':self.pwd,'remember':'true'})
				else:
					self.socklib.connect(self.url+':8080',{'name':self.name,'password':self.pwd,'remember':'true'}) #HTTPS doesn't work with self signed cert in local dev env

			scripts = self.socklib.getScriptList(self.url)
			self.refreshScriptList(scripts)
			print('scripts refreshed')

			self.socklib.joinRoom(self.room,self.roomStatusSignal.emit)
			self.loggedIn = True

			##HANDLERS for remote events
			self.socklib.addHandler('get screenshot',self.getScreenshot.emit)
			self.socklib.addHandler('message',self.interpretMessage)
			self.socklib.addHandler('iot',self.setWidgetValue)
			self.remoteAccessSignal.connect(self.socklib.executeRemoteCallback)#self.remoteAccessCallbacks)
			self.getScreenshot.connect(self.screenshot)
			if self.pname:
				self.socklib.transmitWidgets(layoutname=self.pname)			

		except Exception as e:
			self.loggedIn = False
			print(e)
			self.statusBar.showMessage('connect failed:'+str(e),3000)

	def joinRoomScreen(self):
		self.room = 'd'
		self.joinRoomDialog = utilities.joinDialog(self,classroom = self.room)
		
		retval = self.joinRoomDialog.exec_()
		if not retval:
			return
		self.room = self.joinRoomDialog.getClassroom()
		self.socklib.joinRoom(self.room,self.roomStatusSignal.emit)


	def join_room_callback(self,room,args):
		print(args)
		if 'status' in  args:
			if args['status'] == 'false':
				QtWidgets.QMessageBox.warning(self,'Connection error!', args['message'])
				self.statusBar.showMessage(args['message'],3000)
			elif args['status'] == 'true':
				QtWidgets.QMessageBox.information(self, 'Success', args['message'])
				self.statusBar.showMessage(args['message'],3000)
				self.socklib.activeroom = room
				self.room=room
				self.setWindowTitle('CSpark Remote Labs: %s | %s'%(self.name,self.room))
		else:
			QtWidgets.QMessageBox.warning(self, 'Server Error','Failed to join/leave classroom. Is https://expeyes.scischool.in active?')
			self.statusBar.showMessage('Failed to join/leave classroom',3000)

	def leaveRoomScreen(self):
		self.socklib.leaveRoom(self.room,self.roomStatusSignal.emit)



	def setupBrowser(self):
		## HELP
		self.hwin = helpWin(self)
		self.webLayout.addWidget(self.hwin)
		
		try:
			from PyQt5.QtWebChannel import QWebChannel
			self.channel = QWebChannel()
			self.webhandler = self.rtcHandler(self.socklib,self.url,self.appdir)
			self.channel.registerObject('handler', self.webhandler)
			self.hwin.page().setWebChannel(self.channel)

			self.webhandler.importLibrarySignal.connect(self._downloadAndExecuteExperiment)
			self.webhandler.iotSignal.connect(self.setWidgetValue)
			
			print('main thread is: ',int(QtCore.QThread.currentThreadId()) )
			print('web channel established')
		except Exception as e:
			print('what!',e)
		self.hwin.loadPage(self.url+"/scripts")

		self.statustimer = QtCore.QTimer()
		self.statustimer.timeout.connect(self.sendStatus)
		self.statustimer.start(30)


	##############################
	def setTheme(self,theme):
		self.setStyleSheet("")
		self.setStyleSheet(open(os.path.join(os.path.dirname(os.path.realpath(__file__)),'themes',theme+".qss"), "r").read())

	def setAutoGraph(self,state):
		self.autographtransmit = state

	def _downloadAndExecuteExperiment(self,path,fname,packagehash,filetype):
		self.downloadAndExecuteExperiment(path,fname,packagehash,filetype = filetype)

	def downloadAndExecuteExperiment(self,path,fname,packagehash,**kwargs):
		self.autotransmit = False
		modname = re.split(r'[\\,/]',path)[-1]
		modzip = modname+'.zip'
		modpath = os.path.join(self.appdir,modzip)
		self.statusBar.showMessage('Checking if lastest package exists: '+modpath,200)
		try:
			ziphash = hashlib.md5(open(modpath,"rb").read()).hexdigest()
			if (packagehash == ziphash):
				self.statusBar.showMessage('Module found:'+ ' Launch : '+fname,2000)
				self.loadExperiment(fname,os.path.join(self.appdir,modname),**kwargs)
				return packagehash
			else:
				self.statusBar.showMessage('Module has changed. Downloading... ' + packagehash)

		except IOError:
			self.statusBar.showMessage('module not found. Download... '+modname)

		#This is a good place to check if we already have the file or not. add versioning.
		r = requests.get(self.url+'/'+path+'.zip',verify=False) 
		if r.status_code == 200:
			with open(modpath, 'wb') as f:
				f.write(r.content)
			with zipfile.ZipFile(modpath, 'r') as zip_ref:
				zip_ref.extractall(self.appdir)
			
			self.loadExperiment(fname,os.path.join(self.appdir,modname) , **kwargs )
			self.statusBar.showMessage('Launched experiment : '+fname,2000)
		else:
			self.statusBar.showMessage('file not found: '+	self.url+'/'+path+'/'+fname )

		return hashlib.md5(open(modpath,"rb").read()).hexdigest()

	def _loadExperiment(self,fname,path,filetype):
		self.loadExperiment(fname,path, filetype = filetype)

	def loadExperiment(self,fname,path,**kwargs):
		if self.loggedIn:
			try:
				self.socklib.disconnectGraphConnections()
			except:
				pass

		import sys,importlib
		sys.path.append(path)
		with open(os.path.join(path,fname)) as f:
			self.codeBrowser.setText(f.read())
		f.close()


		if self.newcode:
			self.newcode.setParent(None)
			self.newcode.deleteLater()
			time.sleep(0.5)
			self.newcode = None
		
		filetype = kwargs.get('filetype',None)
		
		if filetype == 'eyes17':
			if not self.p:
				## Connect to device
				try:
					import eyes17.eyes
					self.p = eyes17.eyes.open()
				except Exception as e:
					print('getting device failed. not connected?',e)
					self.p = utilities.eyes17dummy()

			self.pname = fname.split('.')[0] #Get rid of the .py extension
			mylib = importlib.import_module(self.pname)
			self.newcode = mylib.Expt(self.p)

			self.exptLayout.addWidget(self.newcode)
			self.myTabs.setCurrentWidget(self.remoteExptTab)
			self.prepareRemoteMonitorWidgetList(self.remoteExptTab)
			if self.loggedIn:
				self.socklib.set_device_id(self.p.version)
				self.socklib.transmitWidgets(layoutname=self.pname)
			
			self.autotransmit = True

		elif filetype == 'kuttypy':
			if self.p:
				try:
					self.p.close()()
				except Exception as e:
					print('Failed to close',e)

			self.pname = fname.split('.')[0] #Get rid of the .py extension
			mylib = importlib.import_module(self.pname)
			self.newcode = mylib.runWindowed()

			self.exptLayout.addWidget(self.newcode)
			self.myTabs.setCurrentWidget(self.remoteExptTab)
			self.prepareRemoteMonitorWidgetList(self.remoteExptTab)
			if self.loggedIn:
				self.socklib.set_device_id(self.newcode.VERSION)
				print(self.pname,self.socklib.monitorWidgets)
				self.socklib.transmitWidgets(layoutname=self.pname)
			
			self.autotransmit = True



		elif filetype == 'runscript':
			self.pname = fname.split('.')[0] #Get rid of the .py extension
			mylib = importlib.import_module(self.pname)
			self.myTabs.setCurrentWidget(self.codeTab)

		else:
			self.statusBar.showMessage('Script type unsupported',1000)
			return


	def prepareRemoteMonitorWidgetList(self,location):
		self.socklib.clearMonitors()
		self.socklib.addMonitors(location.findChildren((QtGui.QCheckBox,QtGui.QPushButton,QtGui.QSlider,QtGui.QLabel,pg.PlotWidget,QtGui.QStackedWidget,QtGui.QComboBox)),remote=True)
		#self.socklib.addMonitorSpacer('spacer1')

	class rtcHandler(QtCore.QObject):
		peer_id = None
		iceCandidate = QtCore.pyqtSignal(int,str)
		sessionDescription = QtCore.pyqtSignal(str,str)
		makeVideoCall = QtCore.pyqtSignal(bool,bool)
		getInfo = QtCore.pyqtSignal()
		importLibrarySignal = QtCore.pyqtSignal(str,str,str,str)
		sendDataSignal = QtCore.pyqtSignal(str)
		iotSignal = QtCore.pyqtSignal(object)
		def __init__(self,socklib,url,localdir):
			super(AppWindow.rtcHandler,self).__init__()
			print('rtc thread is: ',int(QtCore.QThread.currentThreadId()) )
			self.socklib = socklib
			self.socklib.addHandler('sessionDescription',self.gotSessionDescription)
			self.socklib.addHandler('iceCandidate',self.gotIceCandidate)
			self.socklib.addHandler('init video exchange',self.beginCall)

			self.url = url
			self.appdir = localdir
			self.dataChannel = False
			self.dataChannelFull = False

		@QtCore.pyqtSlot(str,str,str,str)
		def fetchFile(self,path,fname,packagehash,filetype):
				self.importLibrarySignal.emit(path,fname,packagehash,filetype )


		@QtCore.pyqtSlot(str)
		def printInfo(self,info):
			print('INFO: ',info)

		@QtCore.pyqtSlot(str)
		def gotRemoteData(self,info):
			self.iotSignal.emit(json.loads(info))

		@QtCore.pyqtSlot(int,str)
		def relayICECandidate(self,sdpMLineIndex,candidate):
			print('ICE',sdpMLineIndex,candidate)
			self.socklib.sio.emit('relayICECandidate',{'peer_id':self.peer_id,'ice_candidate':{'sdpMLineIndex':sdpMLineIndex,'candidate':candidate} },namespace='/classroom')

		@QtCore.pyqtSlot(str,str)
		def relaySessionDescription(self,tp,sdp):
			print('relaying session description...',tp,sdp[:10])
			self.socklib.sio.emit('relaySessionDescription',{'peer_id':self.peer_id,'session_description':{'type':tp,'sdp':sdp} },namespace='/classroom')

		@QtCore.pyqtSlot()
		def callTeacher(self):
			print('Calling the teacher')
			self.socklib.sio.emit('call the teacher',{ },namespace='/classroom')

		def beginCall(self,config):
			print('begin video call', config)
			self.peer_id = config['peer_id']
			self.makeVideoCall.emit(config['should_create_offer'],config.get('p2pdata',False))
		
		def gotSessionDescription(self,config):
			print('got session description from',config['peer_id'])
			self.sessionDescription.emit(config['session_description']['type'],config['session_description']['sdp'])

		def gotIceCandidate(self,config):
			print('got ice candidate from',config['peer_id'])
			self.iceCandidate.emit(config['ice_candidate']['sdpMLineIndex'],config['ice_candidate']['candidate'])

		#Data Channel
		@QtCore.pyqtSlot(bool)
		def configureDataChannel(self,state):
			self.dataChannel = state
			print('Data channel update..: ',state)

		@QtCore.pyqtSlot(bool)
		def dataChannelBufferFull(self,state):
			self.dataChannelFull = state
			#print('Data channel buffer full state..: ',state)

		def sendData(self,data):
			if self.dataChannel:
				if not self.dataChannelFull:
					self.sendDataSignal.emit(data)
					return True
				elif 'iot' in data: #IOT data is simply streaming the latest state. can be skipped if data buffer is running full
					return True
			return False

	def setWidgetValue(self,data):
		if('closeapp' in data): #TODO remove
			self.connected = False
			self.loggedIn = False
			self.closeEvent(None)#QtWidgets.qApp.quit()
			return
		self.remoteAccessSignal.emit(data)

	'''
	def remoteAccessCallbacks(self,data):
		self.socklib.executeRemoteCallback(data)
	'''
	def closeEvent(self, evnt):
		global app
		try:
			evnt.ignore()
		except:
			pass

		try:
			self.socklib.sio.emit('leave',{'room':self.room},namespace='/classroom')
			time.sleep(0.5)
			self.socklib.sio.disconnect()
		except Exception as e:
			print(e)
		print('goodbye')
		app.exit()

	def interpretMessage(self,data):
		if 'chatmsg' in data:
			self.statusBar.showMessage('Admin: ' + data.get('chatmsg'))			
		elif 'autotransmit' in data:
			self.autotransmit = data.get('autotransmit')
		elif 'autographtransmit' in data:
			self.autographtransmit = data.get('autographtransmit')
		elif 'fetchall' in data:
			self.sendStatus()
		elif 'modify widget list' in data:
			l = data['modify widget list']
			print('Got modified widget list from Remote',l)
			self.loggedIn = False ; time.sleep(0.5)
			activeWidgets = list(self.socklib.monitorWidgets.keys())
			for active in activeWidgets:
				if (active not in l) and self.socklib.monitorWidgets[active]['widget'] != 'graph':
					self.socklib.monitorWidgets.pop(active)
			self.loggedIn = True

	def sendChatMessage(self):
		text = self.chatbox.displayText()
		self.socklib.sendInformation({'chatmsg':text})
		self.chatbox.setText('')

	def sendStatus(self):
		if (not self.loggedIn) or (not self.autotransmit):
			return
		v = self.socklib.getWidgetValues()
		if v!= self.lastTransmittedData:
			self.lastTransmittedData = v
			data = v
		else:
			data = {}

		if not self.webhandler.sendData(json.dumps(data)): 
			self.socklib.transmitWidgetValues(data)   ##Failed to send data via datachannel (Channel closed) , so use regular channel
			self.statustimer.stop()				
			self.statustimer.start(300) #Slow timer for relay through server mode
		else: #Data sent successfully
			self.statustimer.stop()				
			self.statustimer.start(300) #Speed up data rate for P2P

	
	def screenshot(self,args):
		path='tmp.png'#, _filter  = QtWidgets.QFileDialog.getSaveFileName(self, 'Save File', '~/')
		t= time.time()
		#screen = QtWidgets.QApplication.primaryScreen()
		#screenshot = screen.grabWindow( self.winId() )
		#screenshot.save(path)

		pix = QtGui.QPixmap(self.remoteExptTab.size())
		self.remoteExptTab.render(pix)
		pixs = pix.scaledToWidth(int(args.get('width',800)), mode=QtCore.Qt.FastTransformation)
		pixs.save(path)

		#pix=widget.grab()
		#pix.save("save.png")


		with open(path, "rb") as img_file:
		    imgstr = base64.b64encode(img_file.read())
		print('transmitting screenshot')
		self.socklib.sendInformation({'screenshot':"data:image/png;base64,"+imgstr.decode('utf-8')})

	def rawscreenshot(self,args):
		path='tmp.png'#, _filter  = QtWidgets.QFileDialog.getSaveFileName(self, 'Save File', '~/')
		t= time.time()
		screen = QtWidgets.QApplication.primaryScreen()
		screenshot = screen.grabWindow( self.winId() )
		screenshot.save(path)

		with open(path, "rb") as img_file:
		    imgstr = base64.b64encode(img_file.read())
		print('transmitting screenshot')
		self.socklib.sendInformation({'screenshot':"data:image/png;base64,"+imgstr.decode('utf-8')})




def run():
	global app
	print('QT Version',QtCore.__file__)
	app = QtGui.QApplication(sys.argv)
	myapp = AppWindow(app)
	myapp.show()
	sys.exit(app.exec_())

def runonline():
	global app
	print('QT Version',QtCore.__file__)
	app = QtGui.QApplication(sys.argv)
	myapp = AppWindow(app,remote=True)
	myapp.show()
	sys.exit(app.exec_())


if __name__ == "__main__":
	run()



import requests, socketio

loginurl = "http://localhost:8080/login"
payload ={'email':'a@a.com','password':'a','remember':'true'}

session = requests.session()
r = session.post(loginurl, payload)
if r.status_code == 200:
	print(session.cookies.get_dict())
	token = 'remember_token='
	token += session.cookies['remember_token']
	token += '; session='
	token += session.cookies['session']
	
# standard Python
sio = socketio.Client()

sio.connect('http://localhost:8080',headers={'Cookie': token}, namespaces=['/classroom'])




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'loginDialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(402, 199)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/myIcons/plus.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Dialog.setWindowIcon(icon)
        Dialog.setStyleSheet("color: rgb(46, 52, 54);")
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.usernameBox = QtWidgets.QLineEdit(Dialog)
        font = QtGui.QFont()
        font.setPointSize(13)
        self.usernameBox.setFont(font)
        self.usernameBox.setMaxLength(100)
        self.usernameBox.setObjectName("usernameBox")
        self.gridLayout.addWidget(self.usernameBox, 2, 1, 1, 1)
        self.message_3 = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(13)
        self.message_3.setFont(font)
        self.message_3.setObjectName("message_3")
        self.gridLayout.addWidget(self.message_3, 2, 0, 1, 1)
        self.classroomBox = QtWidgets.QLineEdit(Dialog)
        font = QtGui.QFont()
        font.setPointSize(13)
        self.classroomBox.setFont(font)
        self.classroomBox.setMaxLength(100)
        self.classroomBox.setObjectName("classroomBox")
        self.gridLayout.addWidget(self.classroomBox, 6, 1, 1, 1)
        self.message_4 = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(13)
        self.message_4.setFont(font)
        self.message_4.setObjectName("message_4")
        self.gridLayout.addWidget(self.message_4, 6, 0, 1, 1)
        self.message = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(13)
        font.setItalic(True)
        self.message.setFont(font)
        self.message.setObjectName("message")
        self.gridLayout.addWidget(self.message, 0, 0, 1, 3)
        self.message_2 = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(13)
        self.message_2.setFont(font)
        self.message_2.setObjectName("message_2")
        self.gridLayout.addWidget(self.message_2, 3, 0, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 7, 1, 1, 1)
        self.passwordBox = QtWidgets.QLineEdit(Dialog)
        font = QtGui.QFont()
        font.setPointSize(13)
        self.passwordBox.setFont(font)
        self.passwordBox.setMaxLength(100)
        self.passwordBox.setEchoMode(QtWidgets.QLineEdit.PasswordEchoOnEdit)
        self.passwordBox.setObjectName("passwordBox")
        self.gridLayout.addWidget(self.passwordBox, 3, 1, 1, 1)
        self.urlBox = QtWidgets.QLineEdit(Dialog)
        self.urlBox.setObjectName("urlBox")
        self.gridLayout.addWidget(self.urlBox, 1, 0, 1, 2)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept)
        self.buttonBox.rejected.connect(Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)
        Dialog.setTabOrder(self.usernameBox, self.passwordBox)
        Dialog.setTabOrder(self.passwordBox, self.classroomBox)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Login To ExpEYES Online"))
        self.usernameBox.setPlaceholderText(_translate("Dialog", "Enter Username..."))
        self.message_3.setText(_translate("Dialog", "Username"))
        self.classroomBox.setPlaceholderText(_translate("Dialog", "Classroom Name"))
        self.message_4.setText(_translate("Dialog", "Classroom"))
        self.message.setText(_translate("Dialog", "Enter Credentials"))
        self.message_2.setText(_translate("Dialog", "Password"))
        self.passwordBox.setPlaceholderText(_translate("Dialog", "Enter Password"))
from . import res_rc



import setuptools, platform, distutils, sys, os
from distutils.cmd import Command
from distutils.util import execute
from setuptools.command.install import install
import atexit

import shutil
from subprocess import call


with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

def make_rules():
	if 'sdist' in sys.argv: return
	description = "install udev or devd rules (pip must be run as root)"

	udev_reload = ['udevadm', 'control', '--reload-rules']
	udev_trigger = ['udevadm', 'trigger', '--subsystem-match=usb',
		  '--attr-match=idVendor=1A86', '--action=add']
	devd_restart = ['service', 'devd', 'restart']
	linux_create_group = ['groupadd', 'dialout']
	fbsd_create_group = ['pw', 'groupadd', 'sispmctl']

	print('installing rules for accessing hardware...', sys.argv)
	system = platform.system()
	if os.getuid() != 0 and system in ['Linux', 'FreeBSD']:
		raise OSError(
				'You must have root privileges to install udev/devd rules! sudo pip3 install kuttyPy')
	if system == 'Linux':
		shutil.copy('resources/99-eyes17-pip.rules', '/etc/udev/rules.d/')
		execute(lambda: call(linux_create_group), [],
				"Creating dialout group")
		execute(lambda: call(udev_reload), [], "Reloading udev rules")
		execute(lambda: call(udev_trigger), [], "Triggering udev rules")
	elif system == 'FreeBSD':
		shutil.copy('resources/sispmctl.conf', '/usr/local/etc/devd/')
		execute(lambda: call(fbsd_create_group), [],
				"Creating sispmctl group")
		execute(lambda: call(devd_restart), [], "Restarting devd")
	else:
		print("Not Linux, nothing to do.")


class InstallCommand(install):
	def run(self):
		print('works?')
		install.run(self)
		make_rules()

#if 'sdist' not in sys.argv:
atexit.register(make_rules)

setuptools.setup(
    name="eyesonline",
    version="1.0.7",
    install_requires=requirements,
    author="Jithin B.P",
    author_email="jithinbp@gmail.com",
    description="Python package for Remote Access of Eyes17 and other hardware",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/csparkresearch/expeyes17",
    packages=setuptools.find_packages(exclude=['contrib', 'tests']),
    package_data = {
        '': ['*.qss','*.rules','*.conf','*.html','*.png','*.jpg','*.qm'],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        'Intended Audience :: Education',
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
    'console_scripts': [
        'eyesonline=eyesonlineui.app:runonline',
        'eyeslocal=eyesonlineui.app:run',
    		],
	},
	cmdclass = {'install':InstallCommand},
    keywords = 'expeyes17 data-acquisition remote-access virtual-lab lab-at-home oscilloscope waveform-generator sensors physics',
    python_requires='>=3',
    project_urls={  # Optional
        'Source': 'https://github.com/expeyes/expeyes-programs',
        'Read The Docs': 'https://expeyes.in',
        'Buy Hardware': 'https://csparkresearch.in/expeyes17',
    },
)



from PyQt5 import QtGui,QtCore,QtWidgets
from .templates import ui_loginDialog , ui_joinDialog , ui_exptEntry, ui_exptEntryHeader
import numpy as np
import time, hashlib, requests, re, os, zipfile

class loginDialog(QtWidgets.QDialog, ui_loginDialog.Ui_Dialog):
	def __init__(self, parent,**kwargs):
		super(loginDialog, self).__init__(parent, QtCore.Qt.WindowStaysOnTopHint)
		self.setupUi(self)
		self.usernameBox.setText(kwargs.get('username',''))
		self.passwordBox.setText(kwargs.get('password',''))
		self.classroomBox.setText(kwargs.get('classroom',''))
		self.urlBox.setText(kwargs.get('url',''))
	def getUsername(self):
		return self.usernameBox.text()
	def getPassword(self):
		return self.passwordBox.text()
	def getClassroom(self):
		return self.classroomBox.text()
	def getUrl(self):
		return self.urlBox.text()

class joinDialog(QtWidgets.QDialog, ui_joinDialog.Ui_Dialog):
	def __init__(self, parent,**kwargs):
		super(joinDialog, self).__init__(parent, QtCore.Qt.WindowStaysOnTopHint)
		self.setupUi(self)
		self.classroomBox.setText(kwargs.get('classroom',''))
	def getClassroom(self):
		return self.classroomBox.text()


class exptEntry(QtWidgets.QWidget,ui_exptEntry.Ui_Form):
	def __init__(self,fname,intro, path, appdir,packagehash,callback, **kwargs):
		super(exptEntry, self).__init__()
		self.setupUi(self)
		self.fname  = fname
		self.appdir = appdir
		self.intro = intro
		self.path = path
		self.callback = callback
		self.packagehash = packagehash
		self.exptName.setText(fname)
		self.exptIntro.setText(intro)
		self.filetype = kwargs.get('filetype',None)
		
	def play(self):
		self.packagehash = self.callback(self.path,self.fname,self.packagehash,filetype = self.filetype)

class exptEntryHeader(QtWidgets.QWidget,ui_exptEntryHeader.Ui_Form):
	def __init__(self,pkgname, path, device):
		super(exptEntryHeader, self).__init__()
		self.setupUi(self)
		self.pkgname  = pkgname
		self.path = path
		self.device  = device
		self.exptName.setText(pkgname)
		self.exptPath.setText(path)
		self.exptDev.setText(device)


class eyes17dummy():
	version = 'dummy eyes17 time:date'
	connected = True
	def get_voltage(self,chan):
		return (np.random.random() - .5)*5
	def set_sqr1(self,val):
		return val
	def get_voltage_time(self,chan):
		return time.time(),(np.random.random() - .5)*5
	def set_state(self,**kwargs):
		pass
	def save(self,data,fname):
		print('dump to ',fname,data)
		



from flask import json
from PyQt5 import QtWidgets
from collections import OrderedDict
import socketio,requests,functools
class socketController():
	sio = socketio.Client()
	monitorWidgets=OrderedDict()
	remoteAccessWidgetSetter = OrderedDict()
	monitorWidgetsCallbacks = {}
	parent = None
	activeroom = None
	autotransmit = False
	graphWidgetValues = {}
	graphConnections = [] #Signals to autoupdate graphs
	def __init__(self, parent):
		self.parent  = parent
	
	def connect(self,homeurl, creds):
		session = requests.session()
		#creds.update({'myname':'j'})
		r = session.post(homeurl+'/login', creds, verify=False)
		if r.status_code == 200:
			token = 'remember_token='
			token += session.cookies['remember_token']
			token += '; session='
			token += session.cookies['session']
			self.sio.connect(homeurl,headers={'Cookie': token}, namespaces=['/classroom'])
			return self.sio
		else:
			print('invalid status code returned on login',r.status_code)

		return None

	def addHandler(self,event,function):
		self.sio.on(event, function, namespace='/classroom')

	def joinRoom(self,room,callback,**kwargs):
		data = {'room':room}
		data['widgets'] = self.monitorWidgets
		data.update(kwargs)
		self.sio.emit('join',data,namespace='/classroom',callback = functools.partial(callback,room) )

	def leaveRoom(self,room,callback,**kwargs):
		data = {'room':room}
		data['widgets'] = self.monitorWidgets
		data.update(kwargs)
		self.sio.emit('leave',data,namespace='/classroom',callback = functools.partial(callback,None) )


	def transmitWidgets(self,**kwargs):
		if not self.activeroom:return

		data = {'room':self.activeroom}
		data['widgets'] = self.monitorWidgets
		data.update(kwargs)
		self.sio.emit('widget list',data,namespace='/classroom')


	def set_device_id(self,devID):
		'''
		this will set the device ID(versio) in the database
		'''
		self.sio.emit('set device id',{'id':devID},namespace='/classroom')

	def sendInformation(self,info):
		'''
		this will be relayed to the owners of all the rooms this user is in
		'''
		self.sio.emit('chatter',info,namespace='/classroom')


	def addMonitors(self,widgets,**kwargs):
		'''
		Widgets which the teacher can monitor in real time
		'''
		for a in widgets:
			self.addMonitor(a,**kwargs)

	def setLabelText(self,label,data):
		label.setText(data['value'])
	def setCbxState(self,cbx,data):
		cbx.setChecked(data['value'])
	def setSliderValue(self,sldr,data):
		sldr.setValue(int(data['value']))
	def setComboState(self,combo,data):
		combo.setCurrentIndex(int(data['value']))
	def animateButtonClick(self,btn,data): #data param ignored for consistency
		btn.animateClick()
	def setDioState(self,dio,data):
		dio.setState(data['value'])

	def addMonitor(self,a,**kwargs):
		'''
		Widgets which the teacher can monitor in real time
		'''
		if (len(a.objectName())==0 ):
			return
		
		if isinstance(a,QtWidgets.QLabel):
			self.monitorWidgets[a.objectName()]={'widget':'label','text':a.text()}
			self.remoteAccessWidgetSetter[a.objectName()] = functools.partial(self.setLabelText,a)

		elif isinstance(a,QtWidgets.QCheckBox):
			self.monitorWidgets[a.objectName()]={'widget':'checkbox','text':a.text()}
			self.remoteAccessWidgetSetter[a.objectName()] = functools.partial(self.setCbxState,a)

		elif isinstance(a,QtWidgets.QSlider):
			self.monitorWidgets[a.objectName()]={'widget':'slider','value':a.value(),'min':a.minimum(),'max':a.maximum()}
			self.remoteAccessWidgetSetter[a.objectName()] = functools.partial(self.setSliderValue,a)

		elif isinstance(a,QtWidgets.QComboBox):
			self.monitorWidgets[a.objectName()]={'widget':'combobox','value':[a.itemText(i) for i in range(a.count())]}
			self.remoteAccessWidgetSetter[a.objectName()] = functools.partial(self.setComboState,a)

		elif isinstance(a,QtWidgets.QPushButton):
			self.monitorWidgets[a.objectName()]={'widget':'button','text':str(a.text())}	
			self.remoteAccessWidgetSetter[a.objectName()] = functools.partial(self.animateButtonClick,a)

		elif 'PlotWidget' in str(a.__class__):
			self.monitorWidgets[a.objectName()]={'widget':'graph','xlabel':a.plotItem.getAxis('bottom').labelText,'ylabel':a.plotItem.getAxis('left').labelText}	
			for b in a.plotItem.listDataItems():
				b.sigPlotChanged.connect(functools.partial(self.updatePlot,a.objectName(),b))
				self.graphConnections.append(b)
		elif ".DIO'" in str(a.__class__):			#DIO widget from KuttyPy
			self.monitorWidgets[a.objectName()]={'widget':'dio','value':a.getState()}	
			self.remoteAccessWidgetSetter[a.objectName()] = functools.partial(self.setDioState,a)
		elif ".DIOADC'" in str(a.__class__):			#DIO widget from KuttyPy
			self.monitorWidgets[a.objectName()]={'widget':'dioadc','value':a.getState()}	
			self.remoteAccessWidgetSetter[a.objectName()] = functools.partial(self.setDioState,a)
		elif ".DIOPWM'" in str(a.__class__):			#PWM widget from KuttyPy
			self.monitorWidgets[a.objectName()]={'widget':'diopwm','value':a.getState()}	
			self.remoteAccessWidgetSetter[a.objectName()] = functools.partial(self.setDioState,a)
		else:
			print(' UNKNOWN WIDGET: ',str(a.__class__))
			return False


		if a.property("remote"):
			self.monitorWidgets[a.objectName()].update({'remote':a.property("remote")})

		self.monitorWidgets[a.objectName()].update(kwargs) #Override with any keyword arguments

	def executeRemoteCallback(self,data):
		self.remoteAccessWidgetSetter[data['widget']](data)

	def disconnectGraphConnections(self):
		for a in self.graphConnections:
			print('disconnecting',a)
			a.sigPlotChanged.disconnect()
		self.graphConnections = []
		self.graphWidgetValues = {}


	def graphChanged(self,graph): #This function is called when the graph is altered (range) . It disconnects all signals, and new data items to signals.
		self.disconnectGraphConnections()
		for b in graph.plotItem.listDataItems():
			b.sigPlotChanged.connect(functools.partial(self.updatePlot,graph.objectName(),b, ))
			self.graphConnections.append(b)
		print('something changed with the graph',self.graphConnections)

	def truncateFloats(self,obj,decimals=2):
		return json.loads(json.dumps(obj), parse_float=lambda x: round(float(x), decimals) )

	def updatePlot(self,name,plotDataItem):
		if not self.parent.autographtransmit: return
		if 'PlotDataItem' in str(plotDataItem.__class__):
			if(len(plotDataItem.curve.xData)<=2): return
			self.graphWidgetValues = [self.truncateFloats(plotDataItem.curve.xData.tolist()),self.truncateFloats(plotDataItem.curve.yData.tolist())]
		elif 'PlotCurveItem' in str(plotDataItem.__class__):
			if(len(plotDataItem.xData)<=2): return
			self.graphWidgetValues = [self.truncateFloats(plotDataItem.xData.tolist()),self.truncateFloats(plotDataItem.yData.tolist())]
		self.parent.webhandler.sendData(json.dumps({name:{'widget':'graph','value':self.graphWidgetValues}}))

	def addMonitorSpacer(self,unique_name,classattr=''):
		self.monitorWidgets[unique_name]={'widget':'spacer','value':classattr}	

	def addMonitorVariable(self,unique_name,target,getDataFunction,**kwargs):
		self.monitorWidgets[unique_name]={'widget':target}
		self.monitorWidgets[unique_name].update(kwargs)
		self.monitorWidgetsCallbacks[unique_name] = getDataFunction

	def getWidgetValues(self):
		self.widgetValues = {}
		for a in self.monitorWidgets:
			widgetType = self.monitorWidgets[a]['widget']
			widget = self.parent.findChild(QtWidgets.QWidget, a)
			if widgetType == 'label':
				self.widgetValues[a] = {'widget':widgetType,'text':widget.text()}
			elif widgetType == 'checkbox':
				self.widgetValues[a] = {'widget':widgetType,'value':widget.isChecked(),'text':widget.text()}
			elif widgetType == 'combobox':
				self.widgetValues[a] = {'widget':widgetType,'value':widget.currentIndex(),'text':str(widget.currentText())}
			elif widgetType == 'slider':
				self.widgetValues[a] = {'widget':widgetType,'value':widget.value()}
			elif widgetType == 'button':
				self.widgetValues[a] = {'widget':widgetType,'text':widget.text()}
			elif widgetType == 'combobox':
				self.widgetValues[a] = {'widget':widgetType,'value':widget.currentIndex()}
			elif widgetType in ['dio','dioadc','diopwm']:			#DIO / DIOADC / DIOPWM widget from KuttyPy
				self.widgetValues[a] = {'widget':widgetType,'value':widget.getState()}
			elif widgetType == 'graph':
				graphvals = [] #dummy
				xlen = 0
				for b in widget.plotItem.listDataItems():
					if 'PlotDataItem' in str(b.__class__):
						if(len(b.curve.xData)<=2):
							continue #Invalid xdata. minimum 2 points required.
						if(not len(graphvals)):
							graphvals.append(self.truncateFloats(b.curve.xData.tolist()))
							xlen = len(graphvals[-1])
						graphvals.append(self.truncateFloats(b.curve.yData.tolist()))
						#tval = {"x":b.curve.xData.tolist(),"y":b.curve.yData.tolist()}
					elif 'PlotCurveItem' in str(b.__class__):
						if(len(b.xData)<=2):
							continue
						if(not len(graphvals)):
							graphvals.append(self.truncateFloats(b.xData.tolist()))
							xlen = len(graphvals[-1])
						graphvals.append(self.truncateFloats(b.yData.tolist()))
						#tval = {"x":b.xData.tolist(),"y":b.yData.tolist()}
				self.widgetValues[a] = {'widget':widgetType,'value':graphvals} # graphvals ...[x1,y1,y2,y3...]
			elif widgetType in ['combobox','button','spacer']:
				pass #widgetValues[a] = [widgetType,parent.findChild(QtWidgets.QSlider, a).value()]
			else:
				self.widgetValues[a] = {'widget':widgetType,'value':self.monitorWidgetsCallbacks[a]()} #widget in ['label']
		return self.widgetValues

	def transmitWidgetValues(self,widgetValues,**kwargs):
		if not self.activeroom:return

		data = {'room':self.activeroom}
		data.update(kwargs)

		data['widget values'] = widgetValues
		
		self.sio.emit('widget values',data,namespace='/classroom')

	def clearMonitors(self):
		self.monitorWidgets = OrderedDict()
		self.monitorWidgetsCallbacks = {}

	def getScriptList(self,url):
		#self.sio.emit('script list',{},setScriptList,namespace='/classroom')
		r = requests.get(url+'/getStaticData',{'data':'scripts'} ,verify=False)
		if r.status_code == 200:
			return r.json()
		else:
			print('invalid status code returned on login',r.status_code)
			return {}




import subprocess, multiprocessing
from multiprocessing import Process

my_list = [ 't0', 't1', 't2' ]

if __name__ == '__main__':
    processors = multiprocessing.cpu_count()

    for i in range(len(my_list)):
        if( i < processors ):
             cmd = ["python3", "app.py", my_list[i],"skip"]
             child = subprocess.call( cmd, shell=False )



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'joinDialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(402, 83)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/myIcons/plus.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Dialog.setWindowIcon(icon)
        Dialog.setStyleSheet("color: rgb(46, 52, 54);")
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 3, 1, 1, 1)
        self.classroomBox = QtWidgets.QLineEdit(Dialog)
        font = QtGui.QFont()
        font.setPointSize(13)
        self.classroomBox.setFont(font)
        self.classroomBox.setMaxLength(100)
        self.classroomBox.setObjectName("classroomBox")
        self.gridLayout.addWidget(self.classroomBox, 2, 1, 1, 1)
        self.message_4 = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(13)
        self.message_4.setFont(font)
        self.message_4.setObjectName("message_4")
        self.gridLayout.addWidget(self.message_4, 2, 0, 1, 1)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept)
        self.buttonBox.rejected.connect(Dialog.reject)
        self.classroomBox.returnPressed.connect(Dialog.accept)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Join a Classroom"))
        self.classroomBox.setPlaceholderText(_translate("Dialog", "Classroom Name"))
        self.message_4.setText(_translate("Dialog", "Classroom"))
from . import res_rc
